package com.tanpu.community.service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.tanpu.common.api.CommonResp;
import com.tanpu.common.enums.fund.ProductTypeEnum;
import com.tanpu.community.api.beans.qo.AttachmentDetailVo;
import com.tanpu.community.api.beans.qo.ThemeContentQo;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.vo.feign.FundInfoBaseResp;
import com.tanpu.community.api.beans.vo.feign.ProductInfoVO;
import com.tanpu.community.api.beans.vo.feign.ShortVideoBaseInfoResp;
import com.tanpu.community.api.beans.vo.feign.ZhiboListResp;
import com.tanpu.community.api.beans.vo.feign.course.CourseSimpleResp;
import com.tanpu.community.api.enums.RelTypeEnum;
import com.tanpu.community.dao.entity.community.FileRecordEntity;
import com.tanpu.community.dao.entity.community.TopicEntity;
import com.tanpu.community.feign.course.FeignClientForCourse;
import com.tanpu.community.feign.fatools.FeignClientForFatools;
import com.tanpu.community.feign.product.FeignClientForProducts;
import com.tanpu.community.feign.product.FeignForFund;
import com.tanpu.community.feign.product.FeignForPublicFund;
import com.tanpu.community.feign.tanpuroom.FeignClientForTanpuroom;
import com.tanpu.community.feign.zhibo.FeignClientForZhibo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ProductService {


    @Resource
    private FeignClientForTanpuroom feignClientForTanpuroom;

    @Resource
    private FeignClientForZhibo feignClientForZhibo;

    @Resource
    private FeignClientForProducts feignForProduct;

    @Resource
    private FeignClientForCourse feignForCourse;


    @Resource
    private FeignForFund feignForFund;

    @Resource
    private FeignForPublicFund feignForPublicFund;

    @Resource
    private FeignClientForFatools feignClientForFatools;

    @Resource
    private OSSFileService ossFileService;

    @Resource
    private TopicService topicService;


    public void transferAttachement(ThemeQo themeQo) {
        ArrayList<ThemeQo> themeQos = new ArrayList<>();
        themeQos.add(themeQo);
        transferAttachments(themeQos);
    }


    public void transferAttachments(List<ThemeQo> themeQos) {
        // 短视频信息
        Map<String, ShortVideoBaseInfoResp> shortVideoMap = Maps.newHashMap();
        // 课程信息
        Map<String, ShortVideoBaseInfoResp> curriculumMap = Maps.newHashMap();
        // 新课程信息
        Map<String, CourseSimpleResp> courseMap = Maps.newHashMap();
        // 直播信息
        Map<String, ZhiboListResp> zhiboMap = Maps.newHashMap();
        // 设置产品信息
        Map<String, FundInfoBaseResp> fundMap = Maps.newHashMap();
        // 图片信息
        Map<String, String> imgMap = Maps.newHashMap();
        // 话题标题
        Map<String, String> topicMap = Maps.newHashMap();
        // 批量查询资源信息
        this.getResInfo(shortVideoMap, curriculumMap, courseMap, zhiboMap, fundMap, imgMap, topicMap, themeQos);
        //逐个装入
        setResInfo(themeQos, shortVideoMap, curriculumMap, courseMap, zhiboMap, fundMap, imgMap, topicMap);

    }


    /**
     * 获取资源信息
     *
     * @param shortVideoMap
     * @param curriculumMap
     * @param zhiboMap
     * @param fundMap
     */
    private void getResInfo(Map<String, ShortVideoBaseInfoResp> shortVideoMap,
                            Map<String, ShortVideoBaseInfoResp> curriculumMap,
                            Map<String, CourseSimpleResp> courseMap,
                            Map<String, ZhiboListResp> zhiboMap,
                            Map<String, FundInfoBaseResp> fundMap,
                            Map<String, String> imgMap,
                            Map<String, String> topicMap,
                            List<ThemeQo> themeQos
    ) {
        // 白名单基金id
        List<String> tanpuFundIds = Lists.newArrayList();
        // 私募基金id
        List<String> fundIds = Lists.newArrayList();
        // 理财师导入
        List<String> ifaFundIds = Lists.newArrayList();
        // 公募基金id
        List<String> publicFundIds = Lists.newArrayList();
        // 无净值产品id
        List<String> notNetFundIds = Lists.newArrayList();
        // 短视频id
        List<String> shortVideoIds = Lists.newArrayList();
        // 直播id
        List<String> zhiboIds = Lists.newArrayList();
        // 课程id
        List<String> curriculumIds = Lists.newArrayList();
        // 新课程id
        List<String> courseIds = Lists.newArrayList();
        // 图片id
        List<String> imageIds = Lists.newArrayList();
        // 话题id
        List<String> topicIds = Lists.newArrayList();
        // 获取关联资源id
        if (themeQos != null) {
            themeQos.forEach(o -> {
                if (o.getTopicId() != null) {
                    topicIds.add(o.getTopicId());
                }
            });

            themeQos.forEach(o -> o.getContent().forEach(c -> {
                    if (c.getType().equals(RelTypeEnum.LIVE.type)) {
                        zhiboIds.add(c.getValue());
                    } else if (c.getType().equals(RelTypeEnum.SHORT_VIDEO.type)) {
                        shortVideoIds.add(c.getValue());
                    } else if (c.getType().equals(RelTypeEnum.FUND.type)) {
                        if (c.getProductType() == ProductTypeEnum.PUBLIC.type) {
                            publicFundIds.add(c.getValue());
                        } else if (c.getProductType() == ProductTypeEnum.PRIVATE.type) {
                            fundIds.add(c.getValue());
                        } else if (c.getProductType() == ProductTypeEnum.TAMP.type) {
                            tanpuFundIds.add(c.getValue());
                        } else if (c.getProductType() == ProductTypeEnum.CUSTOMER_IMPORT.type) {
                            ifaFundIds.add(c.getValue());
                        } else if (c.getProductType() == ProductTypeEnum.NOT_NET_PRODUCT.type) {
                            notNetFundIds.add(c.getValue());
                        }

                    } else if (c.getType().equals(RelTypeEnum.VIDEO.type) || c.getType().equals(RelTypeEnum.AUDIO.type)) {
                        curriculumIds.add(c.getValue());
                    } else if (c.getType().equals(RelTypeEnum.SINGLE_IMG.type)) {
                        imageIds.add(c.getValue());
                    } else if (c.getType().equals(RelTypeEnum.NEW_COURSE_SECTION_AUDIO.type) ||
                            c.getType().equals(RelTypeEnum.NEW_COURSE_SECTION_VIDEO.type)) {
                        courseIds.add(c.getValue());
                    }
                })

            );

        }
        if (!CollectionUtils.isEmpty(shortVideoIds)) {
            // 短视频列表
            CommonResp<List<ShortVideoBaseInfoResp>> commonResp = feignClientForTanpuroom.getShortVideoBaseInfo(shortVideoIds);
            if (commonResp.isSuccess() && !CollectionUtils.isEmpty(commonResp.getData())) {
                shortVideoMap.putAll(commonResp.getData().stream().collect(Collectors.toMap(ShortVideoBaseInfoResp::getSourceId, item -> item)));
            }
        }
        if (!CollectionUtils.isEmpty(curriculumIds)) {
            // 课程列表
            List<ShortVideoBaseInfoResp> commonResp = feignClientForFatools.getCurriculumByColumnRelId(curriculumIds);
            if (commonResp != null && !CollectionUtils.isEmpty(commonResp)) {
                curriculumMap.putAll(commonResp.stream().collect(Collectors.toMap(ShortVideoBaseInfoResp::getColumnRelId, item -> item)));
            }
        }
        if (!CollectionUtils.isEmpty(courseIds)) {
            // 新版课程列表
            CommonResp<List<CourseSimpleResp>> commonResp = feignForCourse.getCourseSimpleList(courseIds);
            if (commonResp.isSuccess() & !CollectionUtils.isEmpty(commonResp.getData())) {
                courseMap.putAll(commonResp.getData().stream().collect(Collectors.toMap(CourseSimpleResp::getCourseId, item -> item)));
            }
        }
        if (!CollectionUtils.isEmpty(zhiboIds)) {
            // 直播列表
            CommonResp<List<ZhiboListResp>> commonResp = feignClientForZhibo.simpleList(zhiboIds);
            if (commonResp.isSuccess() && !CollectionUtils.isEmpty(commonResp.getData())) {
                zhiboMap.putAll(commonResp.getData().stream().collect(Collectors.toMap(ZhiboListResp::getId, item -> item)));
            }
        }
        if (!CollectionUtils.isEmpty(imageIds)) {
            // 查询文件
            List<FileRecordEntity> fileRecordEntities = ossFileService.queryByIds(imageIds);
            if (!CollectionUtils.isEmpty(fileRecordEntities)) {
                imgMap.putAll(fileRecordEntities.stream().collect(Collectors
                        .toMap(FileRecordEntity::getFileId, FileRecordEntity::getUrl)));
            }
        }
        if (!CollectionUtils.isEmpty(topicIds)) {
            // 查询话题标题
            List<TopicEntity> topicEntities = topicService.queryByIds(topicIds);
            if (!CollectionUtils.isEmpty(topicEntities)) {
                topicMap.putAll(topicEntities.stream().collect(Collectors
                        .toMap(TopicEntity::getTopicId, TopicEntity::getTopicTitle)));
            }
        }
        this.getFundInfo(tanpuFundIds, fundIds, publicFundIds, ifaFundIds, notNetFundIds, fundMap);
    }

    /**
     * 设置产品信息
     *
     * @param tanpuFundIds
     * @param fundIds
     * @param publicFundIds
     * @param fundMap
     */
    private void getFundInfo(List<String> tanpuFundIds,
                             List<String> fundIds,
                             List<String> publicFundIds,
                             List<String> ifaFundIds,
                             List<String> notNetFundIds,
                             Map<String, FundInfoBaseResp> fundMap) {
        Map<String, FundInfoBaseResp> tampFundMap = null;
        Map<String, FundInfoBaseResp> privateFundMap = null;
        Map<String, FundInfoBaseResp> publicFundMap = null;
        Map<String, FundInfoBaseResp> ifaFundMap = null;
        Map<String, FundInfoBaseResp> notNetFundMap = null;
        if (!CollectionUtils.isEmpty(tanpuFundIds)) {
//            ProductListReq productListReq = ProductListReq.builder()
//                    .fundIds(Lists.newArrayList(tanpuFundIds))
//                    .build();
            // 探普产品
            CommonResp<List<ProductInfoVO>> commonResp = feignForProduct.getProductInfoByIds(tanpuFundIds);
            if (commonResp.isSuccess() && !CollectionUtils.isEmpty(commonResp.getData())) {
                List<FundInfoBaseResp> fundInfoBaseRespList = commonResp.getData().stream().map(item -> {
                    FundInfoBaseResp fundInfoBaseResp = FundInfoBaseResp.builder().build();
                    BeanUtils.copyProperties(item, fundInfoBaseResp);
                    return fundInfoBaseResp;
                }).collect(Collectors.toList());
                tampFundMap = fundInfoBaseRespList.stream().collect(Collectors.toMap(FundInfoBaseResp::getFundId, item -> item));
            }

            if (tampFundMap != null && tampFundMap.size() > 0) {
                fundMap.putAll(tampFundMap);
            }
        }
        // 私募产品
        if (!CollectionUtils.isEmpty(fundIds)) {
            // 私募产品
            CommonResp<List<ProductInfoVO>> fundResult = feignForFund.getProductList(fundIds);
            if (fundResult.isSuccess() && !CollectionUtils.isEmpty(fundResult.getData())) {
                List<FundInfoBaseResp> fundInfoBaseRespList = fundResult.getData().stream().map(item -> {
                    FundInfoBaseResp fundInfoBaseResp = FundInfoBaseResp.builder().build();
                    BeanUtils.copyProperties(item, fundInfoBaseResp);
                    return fundInfoBaseResp;
                }).collect(Collectors.toList());
                privateFundMap = fundInfoBaseRespList.stream().collect(Collectors.toMap(FundInfoBaseResp::getFundId, item -> item));
            }

            if (privateFundMap != null && privateFundMap.size() > 0) {
                fundMap.putAll(privateFundMap);
            }
        }

        // 理财师导入产品
        if (!CollectionUtils.isEmpty(ifaFundIds)) {
            CommonResp<List<ProductInfoVO>> ifaFundResult = feignForFund.getPrivateFundList(ifaFundIds);
            if (ifaFundResult.isSuccess() && !CollectionUtils.isEmpty(ifaFundResult.getData())) {
                List<FundInfoBaseResp> fundInfoBaseRespList = ifaFundResult.getData().stream().map(item -> {
                    FundInfoBaseResp fundInfoBaseResp = FundInfoBaseResp.builder().build();
                    BeanUtils.copyProperties(item, fundInfoBaseResp);
                    return fundInfoBaseResp;
                }).collect(Collectors.toList());
                ifaFundMap = fundInfoBaseRespList.stream().collect(Collectors.toMap(FundInfoBaseResp::getFundId, item -> item));
            }

            if (ifaFundMap != null && ifaFundMap.size() > 0) {
                fundMap.putAll(ifaFundMap);
            }
        }

        if (!CollectionUtils.isEmpty(publicFundIds)) {
            // 公募产品
            CommonResp<List<ProductInfoVO>> publicFundResult = feignForPublicFund.getProductList(publicFundIds);
            if (publicFundResult.isSuccess() && !CollectionUtils.isEmpty(publicFundResult.getData())) {
                List<FundInfoBaseResp> fundInfoBaseRespList = publicFundResult.getData().stream().map(item -> {
                    FundInfoBaseResp fundInfoBaseResp = FundInfoBaseResp.builder().build();
                    BeanUtils.copyProperties(item, fundInfoBaseResp);
                    return fundInfoBaseResp;
                }).collect(Collectors.toList());
                publicFundMap = fundInfoBaseRespList.stream().collect(Collectors.toMap(FundInfoBaseResp::getFundId, item -> item));
            }
            if (publicFundMap != null && publicFundMap.size() > 0) {
                fundMap.putAll(publicFundMap);
            }
        }

        if (!CollectionUtils.isEmpty(notNetFundIds)) {
            // 无净值产品
            CommonResp<List<ProductInfoVO>> notNetInfoResult = feignForFund.getProductListNotNetInfo(notNetFundIds);
            if (notNetInfoResult.isSuccess() && !CollectionUtils.isEmpty(notNetInfoResult.getData())) {
                List<FundInfoBaseResp> fundInfoBaseRespList = notNetInfoResult.getData().stream().map(item -> {
                    FundInfoBaseResp fundInfoBaseResp = FundInfoBaseResp.builder().build();
                    BeanUtils.copyProperties(item, fundInfoBaseResp);
                    return fundInfoBaseResp;
                }).collect(Collectors.toList());
                notNetFundMap = fundInfoBaseRespList.stream().collect(Collectors.toMap(FundInfoBaseResp::getFundId, item -> item));
            }
            if (notNetFundMap != null && notNetFundMap.size() > 0) {
                fundMap.putAll(notNetFundMap);
            }
        }
    }

    /**
     * 设置资源信息
     */
    private void setResInfo(List<ThemeQo> themeQos,
                            Map<String, ShortVideoBaseInfoResp> shortVideoMap,
                            Map<String, ShortVideoBaseInfoResp> curriculumMap,
                            Map<String, CourseSimpleResp> courseMap,
                            Map<String, ZhiboListResp> zhiboMap,
                            Map<String, FundInfoBaseResp> fundMap,
                            Map<String, String> imgUrlMap,
                            Map<String, String> topicMap) {
        for (ThemeQo themeQo : themeQos) {
            if (!StringUtils.isEmpty(themeQo.getTopicId()) && topicMap.containsKey(themeQo.getTopicId())) {
                themeQo.setTopicTitle(topicMap.get(themeQo.getTopicId()));
            }
            if (themeQo.getContent() == null) {
                continue;
            }
            for (ThemeContentQo themeContent : themeQo.getContent()) {
                if (themeContent.getType().equals(RelTypeEnum.LIVE.type)) {
                    if (zhiboMap.containsKey(themeContent.getValue())) {
                        ZhiboListResp zhibo = zhiboMap.get(themeContent.getValue());
                        themeContent.setDetail(AttachmentDetailVo.builder().zhibo(zhibo).build());
                    }
                } else if (themeContent.getType().equals(RelTypeEnum.SHORT_VIDEO.type)) {
                    if (shortVideoMap.containsKey(themeContent.getValue())) {
                        themeContent.setDetail(AttachmentDetailVo.builder()
                                .shortVideo(shortVideoMap.get(themeContent.getValue())).build());
                    }
                } else if (themeContent.getType().equals(RelTypeEnum.VIDEO.type) ||
                        themeContent.getType().equals(RelTypeEnum.AUDIO.type)) {
                    if (curriculumMap.containsKey(themeContent.getValue())) {
                        themeContent.setDetail(AttachmentDetailVo.builder()
                                .curriculum(curriculumMap.get(themeContent.getValue())).build());
                    }
                } else if (themeContent.getType().equals(RelTypeEnum.NEW_COURSE_SECTION_VIDEO.type) ||
                        themeContent.getType().equals(RelTypeEnum.NEW_COURSE_SECTION_AUDIO.type)) {
                    if (courseMap.containsKey(themeContent.getValue())) {
                        themeContent.setDetail(AttachmentDetailVo.builder()
                                .course(courseMap.get(themeContent.getValue())).build());
                    }
                } else if (themeContent.getType().equals(RelTypeEnum.FUND.type)) {
                    if (fundMap.containsKey(themeContent.getValue())) {
                        FundInfoBaseResp product = fundMap.get(themeContent.getValue());
                        // 推荐语
                        product.setRemark(themeContent.getRemark());
                        // 产品类型
                        product.setProductType(themeContent.getProductType());
                        themeContent.setDetail(AttachmentDetailVo.builder()
                                .product(product).build());
                    }
                } else if (themeContent.getType().equals(RelTypeEnum.SINGLE_IMG.type)) {
                    if (imgUrlMap.containsKey(themeContent.getValue())) {
                        themeContent.setValue(imgUrlMap.get(themeContent.getValue()));
                    }
                }
            }
        }
    }

}
